/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Contributor(s):
 *    Devon Jensen <velcrospud@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function foxytunesCharsetStringToUnicode(string, srcCharset) {
	gFoxyTunesUnicodeConverter.charset=srcCharset;
	return gFoxyTunesUnicodeConverter.ConvertToUnicode(string);
}

function foxytunesIsNumber(x){
	return parseInt(x, 10) == x;
}

function foxytunesSeconds2TimeStr(length) {
        if (!foxytunesIsNumber(length)) {
		return null;
	}

  	if (length == 4294967) {
		length = 0; // winamp
	}

	if (length == 4294967295) {
		length = 0; // QCD
        }

	var length_hours = Math.floor(length / (60*60));
	length -= length_hours*60*60;
	var length_mins =  Math.floor(length / 60);
	var length_secs =  length - length_mins*60;
	if (length_secs < 10) {
		length_secs = '0' + length_secs;
	}
	var timeStr = length_mins + ':' + length_secs;
	if (length_hours > 0) {
		timeStr = length_hours + ':' + timeStr;
	}
	return timeStr;
}

function foxytunesCompareCharSets(a,b) {
	if (a.getAttribute('label') < b.getAttribute('label')) {
		return -1;
	} else {
		return 1;
	}

}

function foxytunesPopulateCharacterEncodings() {
   var d           = document;
   var popup       = d.getElementById('foxytunes-character-encoding-popup');

   var converterManager = Components.classes["@mozilla.org/charset-converter-manager;1"].getService(Components.interfaces.nsICharsetConverterManager);
   var decoders = converterManager.getDecoderList();

   var i = 0;
   gFoxyTunesAllCharSets = new Array();
   var newElement;
   while (decoders.hasMore()) {
	var charset = decoders.getNext();
	var charsetTitle = '';
	try {
		charsetTitle = converterManager.getCharsetTitle(charset);
	} catch (error) {
        }		

	var charsetName;
	if (charsetTitle == '') {
		charsetName = charset;
	} else {
		charsetName = charsetTitle;
	}
	newElement = d.createElement('menuitem');
	newElement.setAttribute('name', 'foxytunes-all-charsets');
        newElement.setAttribute('label', charsetName);
        newElement.setAttribute('tooltiptext', charsetName);
        newElement.setAttribute('value', charset);
      	newElement.setAttribute('oncommand', 'foxytunesSelectCharset(event)'); 
	gFoxyTunesAllCharSets[i] = newElement; 
	i++;
   }

   gFoxyTunesAllCharSets.sort(foxytunesCompareCharSets);
   for (var i=0; i < gFoxyTunesAllCharSets.length; i++ ) { 
        popup.appendChild(gFoxyTunesAllCharSets[i]);
   }
}

function foxytunesTryGetFoxyTunesPlayer(contract) {
   try {
	var contractArray = contract.split("/");
	if (contractArray[1].toLowerCase() != 'foxytunes') {
		return null;
	}
	var subModule = contractArray[2];
	var subModuleArray = subModule.split(";");
	return subModuleArray[0];
   } catch (err) {
	dump("\nError while parsing contractID: " + contract + " - " + err + "\n");
	return null;
   }
}

function foxytunesPopulatePlayers() {
   var d           = document;
   var popup       = d.getElementById('foxytunes-available-player-popup');
   var newElement;
   var iPlayer = 0;
   var playerMenuItems = new Array();
   for (var classContract in Components.classes) {
	var player = foxytunesTryGetFoxyTunesPlayer(classContract);
	if (player != null) {
    		newElement = d.createElement('menuitem');
       	 	newElement.setAttribute('type', 'radio');
       	 	newElement.setAttribute('name', 'foxytunes-player');
       	 	newElement.setAttribute('label', player);
		    newElement.playerName = player;
	        newElement.setAttribute('tooltiptext', player);
        	newElement.setAttribute('value', classContract);
        	newElement.setAttribute('class', 'menuitem-iconic player-item'); 
        	newElement.setAttribute('playername', player.toLowerCase());
        	newElement.setAttribute('oncommand', 'foxytunesSelectPlayer(event)');
		if ((gFoxyTunesCurrentPlayerClass != null) &&
			(gFoxyTunesCurrentPlayerClass == classContract)) {
			// select the user preferred player
        		newElement.setAttribute('checked', 'true');
		} else {
			if ((gFoxyTunesCurrentPlayerClass == null) && 
				(iPlayer == 0)) {
					// select the first available player
					gFoxyTunesCurrentPlayerClass = classContract;	
					foxytunesWritePreferences();
		        		newElement.setAttribute('checked', 'true');
			}
		}
		playerMenuItems.push(newElement);
		iPlayer++;
	}
   }

   var sortFunc = function (a, b) { 	a = a.playerName.toLowerCase(); 
					b = b.playerName.toLowerCase(); 
					if ( a < b ) return -1;
				        if ( a > b ) return 1;
			        	return 0;   }
   playerMenuItems.sort(sortFunc);
   for (var i = 0; i < playerMenuItems.length; i++) {
	popup.appendChild(playerMenuItems[i]);
   }
   return iPlayer;
}

// ********* start recent charsets stuff *************
function foxytunesGetCharSetMenuItem(charsetList, charset, clone) {
   for (var i=0; i < charsetList.length; i++) {
	if (charsetList[i].getAttribute('value') == charset) {
                if (clone) {
			return charsetList[i].cloneNode(true);
		} else {
			return charsetList[i];
		}
	}
   }
   return null;
}

// remove an element from the recent list, if too long
function foxytunesTrimRecentCharSets(charSetMenu, groupName) {
   var nRecentEntries = 0;
   var children = charSetMenu.childNodes;
   var removeCandidate;
   for (var i=0; i < children.length; i++) {
       if (children[i].getAttribute('name') == groupName) {
          nRecentEntries++;
          if (nRecentEntries == 2) {
		removeCandidate = children[i]; // always remove the second node (the one after default)
          }
       }
   }
   if (nRecentEntries > gFoxyTunesMaxRecentCharsets) {
	charSetMenu.removeChild(removeCandidate);
   }
}

function foxytunesAddRecentCharSet(charset, checked) {
   var groupName = 'foxytunes-character-encoding-recent';
   var charetRecentMenu = document.getElementById('foxytunes-charsets-recent-menu');
   var recentMenuItem = foxytunesGetCharSetMenuItem(charetRecentMenu.childNodes, charset, false);
   if (recentMenuItem != null) {
	// charset already exists in recent
	if (checked) {
      	     recentMenuItem.setAttribute('checked', 'true');
	}
   } else {
   	// clone item from the "all" list:
   	var newRecentMenuItem = foxytunesGetCharSetMenuItem(gFoxyTunesAllCharSets, charset, true);
        if (newRecentMenuItem == null) {
		dump("\naddRecentCharSet: couldn't find charset: " + charset + "\n");
		return;
	}
 	var charSetRecentSeparator = document.getElementById('foxytunes-charsets-recent-separator');
	newRecentMenuItem.setAttribute('name', groupName);
        newRecentMenuItem.setAttribute('type', 'radio');
      	newRecentMenuItem.setAttribute('oncommand', 'foxytunesSelectCharset(event)'); 
	if (checked) {
      	     newRecentMenuItem.setAttribute('checked', 'true');

	}
	charetRecentMenu.insertBefore(newRecentMenuItem, charSetRecentSeparator);
	foxytunesTrimRecentCharSets(charetRecentMenu, groupName);
   }
}

function foxytunesPopulateRecentCharsets() {
  var charsetArray = gFoxyTunesRecentCharsets.split(', ');

  if (charsetArray.length > 0) {
    for (var i = 0; i < charsetArray.length; i++) {
	var isCurrent = charsetArray[i] == gFoxyTunesCharacterEncoding;
	foxytunesAddRecentCharSet(charsetArray[i], isCurrent);
	if (isCurrent) {
		// we have some other recend charset selected, deselect "default":
		document.getElementById('foxytunes-character-encoding-default').setAttribute('checked', 'false');
	}
    }
  }
}

function foxytunesSelectCharset(event) {
    var item = event.target;
    gFoxyTunesCharacterEncoding = item.getAttribute("value");
    foxytunesAddRecentCharSet(gFoxyTunesCharacterEncoding, true);
    foxytunesWritePreferences();

    // update the player
    foxytunesSetPlayerOptions();

    // update the track display
    foxytunesUpdateTrackTitleAfterCommand();
}

function foxytunesUpdateRecentCharsetsList() {
   var groupName = 'foxytunes-character-encoding-recent';
   var charetRecentMenu = document.getElementById('foxytunes-charsets-recent-menu');
   var children = charetRecentMenu.childNodes;
   gFoxyTunesRecentCharsets = '';
   var nAdded = 0;
   for (var i=0; i < children.length; i++) {
       var charSet = children[i].getAttribute('value');
       if ((children[i].getAttribute('name') == groupName) &&
		(charSet != 'default')) {
                if (nAdded == 0) {
			gFoxyTunesRecentCharsets = charSet;
		} else {
			gFoxyTunesRecentCharsets = gFoxyTunesRecentCharsets + ", " + charSet;
		}
		nAdded++;
       }
   }
}

// ********* end   recent charsets stuff *************

function foxytunesWritePreferences() {
   try {
     gFoxyTunesPref.setCharPref("player_class", gFoxyTunesCurrentPlayerClass);
   } 
   catch (err) {
     dump("\nSetting preference for player_class failed: " + err + "\n");
   }
   try {
     gFoxyTunesPref.setCharPref("character_encoding", gFoxyTunesCharacterEncoding);
   } 
   catch (err) {
     dump("\nSetting preference for character_encoding failed: " + err + "\n");
   }

   foxytunesUpdateRecentCharsetsList();
   try {
     gFoxyTunesPref.setCharPref("recent_charsets", gFoxyTunesRecentCharsets);
   } 
   catch (err) {
     dump("\nSetting preference for recent_charsets failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("max_n_recent_charsets", gFoxyTunesMaxRecentCharsets);
   } 
   catch (err) {
     dump("\nSetting preference for max_n_recent_charsets failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("current_player_options", gFoxyTunesCurrentPlayerOptions);
   } 
   catch (err) {
     dump("\nSetting preference for current_player_options failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("custom_player_options", gFoxyTunesCustomPlayerOptions);
   } 
   catch (err) {
     dump("\nSetting preference for custom_player_options failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setBoolPref("open_window_in_tab", gbFoxyTunesOpenWindowInTab);
   } 
   catch (err) {
     dump("\nSetting preference for open_window_in_tab failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("parent_element_id", gFoxyTunesParentElementID);
   } 
   catch (err) {
     dump("\nSetting preference for parent_element_id failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("insert_before_element_id", gFoxyTunesInsertBeforeElementId);
   } 
   catch (err) {
     dump("\nSetting preference for insert_before_element_id failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setCharPref("insert_after_element_id", gFoxyTunesInsertAfterElementId);
   } 
   catch (err) {
     dump("\nSetting preference for insert_after_element_id failed: " + err + "\n");
   }

   try {
     gFoxyTunesPref.setBoolPref("url_unescape", gbURLUnescape);
   } 
   catch (err) {
     dump("\nSetting preference for url_unescape failed: " + err + "\n");
   }


   try {
	gFoxyTunesPrefService.savePrefFile(null);
   }
   catch (err) {
        dump("\nSaving preferences failed: " + err + "\n");
   }

}

function foxytunesReadPreferences() {
   try {
     gFoxyTunesCurrentPlayerClass = gFoxyTunesPref.getCharPref("player_class");
   } 
   catch (err) {
     gFoxyTunesCurrentPlayerClass = null;
   }

   try {
     gFoxyTunesCharacterEncoding = gFoxyTunesPref.getCharPref("character_encoding");
   } 
   catch (err) {
     gFoxyTunesCharacterEncoding = 'default';
   }

   try {
     gFoxyTunesRecentCharsets = gFoxyTunesPref.getCharPref("recent_charsets");
   } 
   catch (err) {
     gFoxyTunesRecentCharsets = '';
   }

   try {
     gFoxyTunesMaxRecentCharsets = gFoxyTunesPref.getCharPref("max_n_recent_charsets");
   } 
   catch (err) {
     gFoxyTunesMaxRecentCharsets = 5;
   }

   try {
     gFoxyTunesCustomPlayerOptions = gFoxyTunesPref.getCharPref("custom_player_options");
   } 
   catch (err) {
     gFoxyTunesCustomPlayerOptions = '';
   }

   try {
     gFoxyTunesCurrentPlayerOptions = gFoxyTunesPref.getCharPref("current_player_options");
   } 
   catch (err) {
     gFoxyTunesCurrentPlayerOptions = '';
   }

   try {
     gbFoxyTunesOpenWindowInTab = gFoxyTunesPref.getBoolPref("open_window_in_tab");
   } 
   catch (err) {
     gbFoxyTunesOpenWindowInTab = false;
   }


   try {
     gFoxyTunesParentElementID = gFoxyTunesPref.getCharPref("parent_element_id");
   } 
   catch (err) {
     gFoxyTunesParentElementID = '';
   }

   try {
     gFoxyTunesInsertBeforeElementId = gFoxyTunesPref.getCharPref("insert_before_element_id");
   } 
   catch (err) {
     gFoxyTunesInsertBeforeElementId = '';
   }

   try {
     gFoxyTunesInsertAfterElementId = gFoxyTunesPref.getCharPref("insert_after_element_id");
   } 
   catch (err) {
     gFoxyTunesInsertAfterElementId = '';
   }


   try {
     gFoxyTunesAutoHideTimeout = gFoxyTunesPref.getIntPref("autohide-timeout");
   } 
   catch (err) {}

   try {
     gbFoxyTunesDontUpdateTitleOnCommand = gFoxyTunesPref.getBoolPref("no_update_title_on_command");
   } 
   catch (err) {
     gbFoxyTunesDontUpdateTitleOnCommand = false;
   }

   try {
     gbFoxyTunesNoTitlePopup = gFoxyTunesPref.getBoolPref("no_title_popup");
   } 
   catch (err) {
     gbFoxyTunesNoTitlePopup = false;
   }

   try {
     gbURLUnescape = gFoxyTunesPref.getBoolPref("url_unescape");
   } 
   catch (err) {
     gbURLUnescape = false;
   }
}

function foxytunesSelectPlayer(event) {
    var item = event.target;
    gFoxyTunesCurrentPlayerClass = item.getAttribute("value");
    foxytunesAddonSetCurrentPlayerIcon();
    foxytunesWritePreferences();
    
    // invalidate the global object. it will be initialized on next operation
    gFoxyTunesPlayerObj = null;
    
    foxytunesUpdateTrackTitleLabel(true);
}

function foxytunesInitPlayerObjectIfNeeded() {
   if (gFoxyTunesPlayerObj != null) {
	return;
   }
   try {
        //gFoxyTunesPlayerObj = Components.classes[gFoxyTunesCurrentPlayerClass].createInstance();
        gFoxyTunesPlayerObj = Components.classes[gFoxyTunesCurrentPlayerClass].getService();
	dump("\nGetting service...\n");
   } 
   catch (err) {
        dump("\nCreateInstance for " + gFoxyTunesCurrentPlayerClass + " failed: " + err + "\n");
	return;
   }

   try {
        gFoxyTunesPlayerObj = gFoxyTunesPlayerObj.QueryInterface(Components.interfaces.mozIFoxyTunes);
   }
   catch (err) {
        dump("\nQueryInterface for " + gFoxyTunesCurrentPlayerClass + " failed: " + err + "\n");
	gFoxyTunesCurrentPlayerClass = null;
	return;
   }

   // mark volume slider as dirty, because we just connected to the player
   foxytunesVolumeSliderPositionDirty("true");

   foxytunesSetPlayerOptions(); // send the options to the player 
}

function foxytunesUpdateTrackTitleAfterCommand() {
	foxytunesUpdateTrackTitleLabel();
	foxytunesUpdateTrackPositionMarker();

        // some players take time to respond:
	if (gFoxyTunesUpdateTitleOnCommandTimerID != 0) {
		clearTimeout(gFoxyTunesUpdateTitleOnCommandTimerID);
	}
        gFoxyTunesUpdateTitleOnCommandTimerID = setTimeout('foxytunesUpdateTrackTitleLabel()', 400);
}

function foxytunesDispatchPlayerCommand(cmd, bUpdateTitle) {
   dump("Dispatching: " + cmd + '\n');
   foxytunesInitPlayerObjectIfNeeded();
   try {
        if (typeof gFoxyTunesPlayerObj[cmd] == 'function') {
            gFoxyTunesPlayerObj[cmd]();
        }
   }
   catch (err) {
        dump("\n" + cmd + " failed: " + err + "\n");
   }

   // if parameter says we should update 
   // and the preferences didn't override this
   if (bUpdateTitle && !gbFoxyTunesDontUpdateTitleOnCommand) { 
	foxytunesUpdateTrackTitleAfterCommand();
   }
}

function foxytunesSetVolume(volume) {
    foxytunesInitPlayerObjectIfNeeded();
    try {
        gFoxyTunesPlayerObj.SetVolume(volume);
    }
    catch (err) {
        dump("\n" + 'SetVolume' + " failed: " + err + "\n");
	return false;
    }
    return true;
}

function foxytunesGetVolume() {
    foxytunesInitPlayerObjectIfNeeded();
    var volume = -1;
    try {
        volume = gFoxyTunesPlayerObj.GetVolume();
    }
    catch (err) {
        dump("\n" + 'GetVolume' + " failed: " + err + "\n");
    }
    return volume;
}

function foxytunesGetCurrentTrackTitle() {
    foxytunesInitPlayerObjectIfNeeded();
    var title = '';
    try {
        title = gFoxyTunesPlayerObj.GetCurrentTrackTitle();
 	if (title == null) {
		title = '';  // if we are here, there was no exception, so the title
			     // is supported and should be blank
	}
    }
    catch (err) {
        dump("\n" + 'GetCurrentTrackTitle' + " failed: " + err + "\n");
        if (err.result == Components.results.NS_ERROR_NOT_IMPLEMENTED) {
		// set the title to null only if function is not implemented
		// some player controllers that do implement the function sometimes
		// return null and we do not want a "not supported" message to the user
		title = null; 
        }

    }
   
    try { 
	if (title != null) {
        if (gbURLUnescape) {
            title = unescape(title);
        }
		if (gFoxyTunesCharacterEncoding != 'default') {
			title = foxytunesCharsetStringToUnicode(title, gFoxyTunesCharacterEncoding);
		} else {
			title = foxytunesCharsetStringToUnicode(title, 'UTF-8');
		}
	}
    } catch (err) { 
	//dump("Couldn't convert title: " + err + '\n');
    }

    // cache the title:
    gFoxyTunesCurrentTrackTitle = title;

    return title;
}

function foxytunesGetCurrentTrackItem(item) {
    foxytunesInitPlayerObjectIfNeeded();
    var itemVal = null;
    try {
        var cmd = 'GetCurrentTrack' + item;
        if (typeof gFoxyTunesPlayerObj[cmd] == 'function') {
            itemVal = gFoxyTunesPlayerObj[cmd]();
        }
    }
    catch (err) {
        dump("\n" + 'GetCurrentTrackItem (' + item + ") failed: " + err + "\n");
    }
    return itemVal;
}

function foxytunesSetCurrentTrackPosition(pos) {
    foxytunesInitPlayerObjectIfNeeded();
    try {
	gFoxyTunesPlayerObj.SetCurrentTrackPosition(pos);
    } 
    catch (err) {
        dump("\n" + 'foxytunesSetCurrentTrackPosition failed: ' + err + "\n");
	return false;
    }
    return true;
}

function foxytunesGetTrackInfoTooltipText() {
    var tipText = foxytunesGetCurrentTrackTitle();
    if (tipText == null) {

	var str = foxytunesGetLocaleString("noTrackInfoAvailable");
	return str;
    }

    var pos = foxytunesGetCurrentTrackItem('Position');
    if (pos != null) {
       pos = foxytunesSeconds2TimeStr(pos);
    }
    var length = foxytunesGetCurrentTrackItem('Length');
    if (length != null) {
       length = foxytunesSeconds2TimeStr(length);
    }
    if (length != null) {
	if ((pos != null) && (pos != '0:00')) {
		    // add LRE character to prevent wrong display in Hebrew
		    tipText += " \u202a (" + pos + " / " + length + ") "; 
		    gbFoxyTunesPlaying = true;
    	} else {
		    if (length != '0:00') {
			    tipText += " \u202a (" + length + ") ";
		    }
		    gbFoxyTunesPlaying = false;
    	}
    } else {
	gbFoxyTunesPlaying = false;
    }

    // do not show just zeros or an empty tooltop
    if ((tipText == " \u202a (" + '0:00' + ") ") || (tipText == '')) {
	    var str = foxytunesGetLocaleString("idle");
	    tipText = str;
    }

    return tipText;
}

function foxytunesSetTrackInfoTooltip() {
  var textNode = document.getElementById("foxytunes-track-info-tooltip");
  textNode.setAttribute('value', foxytunesGetTrackInfoTooltipText());
}

function foxytunesHideAllPopupsByType(type) {
  try {
   var elements = document.getElementsByTagName(type);
   for (var i=0; i < elements.length; i++) {
	elements[i].hidePopup();
   } 
  }
  catch (err) {
      dump("\nHideAllPopups error: " + err + "\n");
  }
}

function foxytunesHideAllPopups(type) {
  foxytunesHideAllPopupsByType('menupopup');
  foxytunesHideAllPopupsByType('popup');
}


function foxytunesShowTrackInfoPopup(event) {
   var popup = document.getElementById('foxytunes-track-info-popup');
   var trackInfoButton = document.getElementById('foxytunes-track-info-statusbar');

   // Firefox crash on Mac, during hide popup https://bugzilla.mozilla.org/show_bug.cgi?id=264657
   if (gbFoxyTunesNoTitlePopup) { 
	var title = foxytunesGetCurrentTrackTitle();
	if (title == '') {
		var str = foxytunesGetLocaleString("idle");
		title = str;
	}

	trackInfoButton.setAttribute('tooltiptext', title);
	return;
   }

   // I must hide all the popups, because if there is a visible popup
   // it will hang on the screen after I call my showPopup (Bug?)
   foxytunesHideAllPopups();

   document.popupNode = trackInfoButton;

   // should be like to following line, causes strange jumps:
   //popup.showPopup(trackInfoButton, -1, -1, "popup", "topleft", "bottomleft");

   var offset;
   if (gFoxyTunesTrackInfoTooltipHeight != 0) {
	offset = gFoxyTunesTrackInfoTooltipHeight; 
   } else {
	offset = trackInfoButton.boxObject.height*2;
   }
 
   popup.showPopup(trackInfoButton, 
		trackInfoButton.boxObject.x, 
		trackInfoButton.boxObject.y - offset, 
		"tooltip", "topleft", "bottomleft");

   if (gFoxyTunesTrackInfoTooltipHeight == 0) {
        // this is the first time we show the popup
	// remember the height for the next time
	// reshow at the correct place
	gFoxyTunesTrackInfoTooltipHeight = popup.boxObject.height + 4;
	popup.showPopup(trackInfoButton, 
		trackInfoButton.boxObject.x, 
		trackInfoButton.boxObject.y - gFoxyTunesTrackInfoTooltipHeight, 
		"tooltip", "topleft", "bottomleft");
   } 

   dump("\nShowing TrackInfoPopup\n"); 
   return true;
}

function foxytunesHideTrackInfoPopup(event) {
   // Firefox crash on Mac, during hide popup https://bugzilla.mozilla.org/show_bug.cgi?id=264657
   if (gbFoxyTunesNoTitlePopup) {
	return;
   } 
   document.getElementById('foxytunes-track-info-popup').hidePopup(); 
   return true;
}

function foxytunesStartTrackInfoTooltip() {
  foxytunesSetTrackInfoTooltip();

  // update the track title display:
  foxytunesSetCurrentTrackTitleLabel(gFoxyTunesCurrentTrackTitle);

  if (gFoxyTunesTrackInfoTimerID != 0) {
	// clear the previous timeout if exists
	// so we don't have multiple timers
	clearTimeout(gFoxyTunesTrackInfoTimerID);
	gFoxyTunesTrackInfoTimerID = 0;
  }

  if (gbFoxyTunesPlaying) {
	  gFoxyTunesTrackInfoTimerID = setTimeout('foxytunesStartTrackInfoTooltip()', 1000);
  } else {
	  gFoxyTunesTrackInfoTimerID = 0;
  }
  return true;
}

function foxytunesEndTrackInfoTooltip() {
  if (gFoxyTunesTrackInfoTimerID != 0) {
	clearTimeout(gFoxyTunesTrackInfoTimerID);
        dump("\nTrackInfoTimer " + gFoxyTunesTrackInfoTimerID + " ended\n");
	gFoxyTunesTrackInfoTimerID = 0;
  } 
  return true;
}

function foxytunesVolumeSliderChanged(event) {
   if ((event.attrName == 'curpos') && !gbFoxyTunesChangingSliderPos) {
	//dump("\nChanging volume to " + event.newValue  + "\n");
	var changed = foxytunesSetVolume(event.newValue);
	if (! changed) {
		// couldn't set volume try increase/decrease
		if (event.newValue > event.prevValue) {
			foxytunesDispatchPlayerCommand('IncreaseVolume');
		} else if (event.newValue < event.prevValue) {
			foxytunesDispatchPlayerCommand('DecreaseVolume');
		}
		// the foxytunesSetVolume method failed, return slider to the middle
		setTimeout('foxytunesSetVolumeSliderPosition(50)', 1000);
	}
   }
   return false;
}

// this function sets the volume slider curpos, 
// but temporarily disables DOMAttrModified event processing 
// it is used when the GUI position of the slider has to be changed, 
// but no SetVolume calls need to be made
function foxytunesSetVolumeSliderPosition(pos) {
  gbFoxyTunesChangingSliderPos = true;  // foxytunesVolumeSliderChanged will ignore the event

  var volumeSlider = document.getElementById('foxytunes-volume-slider'); 
  volumeSlider.setAttribute('curpos', pos);

  gbFoxyTunesChangingSliderPos = false;
}

// set to true if the slider position doesn't reflect the actual volume
// and needs to be update when needed, false if it does
// call without params to get the current dirty value
function foxytunesVolumeSliderPositionDirty(dirty) {
   var volumeSliderWidget = document.getElementById('foxytunes-volume-slider-widget'); 
   if (dirty == null) {
       	return volumeSliderWidget.getAttribute('dirty');
   } else {
  	volumeSliderWidget.setAttribute('dirty', dirty);
	return null;
   }
}

// update the volume slider position with the current volume
function foxytunesRefreshVolumeSliderPosition() {
   // if the slider is visible, update its position
   var volumeSliderWidget = document.getElementById('foxytunes-volume-slider-widget');
   if (volumeSliderWidget.getAttribute('hidden') == 'false') {
	var volume = foxytunesGetVolume();
    //dump('volume ' + volume + "\n");
	if (volume == -1) {		
		// cannot get volume, set the slider to the middle
		volume = 50;
	}
	foxytunesSetVolumeSliderPosition(volume);
        foxytunesVolumeSliderPositionDirty("false");
   }
}

function foxytunesRefreshVolumeSliderIfDirty() {
   if (foxytunesVolumeSliderPositionDirty() == 'true') {
	foxytunesRefreshVolumeSliderPosition();
   } 
}

function foxytunesIncreaseVolume() {
    foxytunesDispatchPlayerCommand('IncreaseVolume');
    foxytunesRefreshVolumeSliderPosition();
}

function foxytunesDecreaseVolume() {
    foxytunesDispatchPlayerCommand('DecreaseVolume');
    foxytunesRefreshVolumeSliderPosition();
}

function foxytunesScrollOnVolumeControls(event) {
   event.preventDefault();
   if (event.detail > 0) {
	foxytunesDecreaseVolume();
   } else {
	foxytunesIncreaseVolume();
   }
}

function foxytunesToggleObjectVisibilityWithArrow(arrow, object, bInverted) {
  var hidden = object.getAttribute('hidden');
  var side;
  if (hidden == 'true') {
	hidden = 'false';
	side = 'left';
  } else {
	hidden = 'true';
	side = 'right';
  }
  if (bInverted) {
	if (side == 'left') {
		side = 'right';
	} else {
		side = 'left';
	}
  }
  arrow.setAttribute('class', side + " foxytunes-statusbar-arrow-button");
  object.setAttribute('hidden', hidden);
}

function foxytunesToggleURLUnescapeTitle(event)
{
  gbURLUnescape = !gbURLUnescape;
  foxytunesWritePreferences();
}

function foxytunesToggleOpenWindowInTab(event)
{
  gbFoxyTunesOpenWindowInTab = !gbFoxyTunesOpenWindowInTab;
  foxytunesWritePreferences();
}

function foxytunesToggleSeparatorsVisibility(event) {
  var menuItem = event.target;
  var bHidden = event.target.getAttribute('checked');
  var foxyTunes = document.getElementById('foxytunes-statusbar-panel');
  var separators = foxyTunes.getElementsByTagName('toolbarseparator');
  for (var i = 0; i < separators.length; i++) {
	separators[i].setAttribute('hidden', bHidden);
  }
}

function foxytunesToggleButtonVisibility(event) {
  var menuItem = event.target;
  var targetButtonID = menuItem.getAttribute('foxytunes-item');
  var targetButton = document.getElementById(targetButtonID);
  targetButton.setAttribute("hidden", menuItem.getAttribute("checked"));
}

function foxytunesToggleVolumeSliderVisibility(event) {
  var arrowButton = event.target; 
  var volumeSlider = document.getElementById('foxytunes-volume-slider-widget'); 
  foxytunesToggleObjectVisibilityWithArrow(arrowButton, volumeSlider);
  foxytunesRefreshVolumeSliderPosition();
}

function foxytunesAllButtonsAutoHideIsOn() {
  var autoHideMenuItem = document.getElementById('foxytunes-main-panel-auto-hide-toggle');
  return (autoHideMenuItem.getAttribute('checked') == 'true');
}

function foxytunesAllButtonsMouseOver() {
  // cancel all timers
  if (gFoxyTunesAllButtonsAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesAllButtonsAutohideTimerID);
  }
}

function foxytunesAllButtonsMouseOut() {
  if (! foxytunesAllButtonsAutoHideIsOn()) {
  	// auto-hide is off, nothing to do
	return;
  }

  var arrowButton = document.getElementById('foxytunes-toggle-all-buttons-container-visibility-statusbar'); 
  var allButtons = document.getElementById('foxytunes-all-buttons-container-statusbar'); 
  var bHidden = (allButtons.getAttribute('hidden') == 'true');

  if (bHidden) {
	// already hidden, nothing to do
	return;
  }

  // stop the previous timers if exist
  if (gFoxyTunesAllButtonsAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesAllButtonsAutohideTimerID);
  }

  // schedule auto-hide
  gFoxyTunesAllButtonsAutohideTimerID = 
			setTimeout(foxytunesToggleObjectVisibilityWithArrow, gFoxyTunesAutoHideTimeout, arrowButton, allButtons, false);
}

function foxytunesToggleAllButtonsVisibility(event, bHovering) {
  var arrowButton = event.target; 
  var allButtons = document.getElementById('foxytunes-all-buttons-container-statusbar'); 
  var bHidden = (allButtons.getAttribute('hidden') == 'true');

  // if hovering and the panel is already visible, or auto-hide is off, return
  if (bHovering && (!bHidden || !foxytunesAllButtonsAutoHideIsOn())) {	
	return;
  }

  foxytunesToggleObjectVisibilityWithArrow(arrowButton, allButtons);
}

function foxytunesOpenFoxyTunesAboutDialog() {
  window.open('chrome://foxytunes/content/about.xul','','chrome,centerscreen');
}

function foxytunesGetAllKeys() {
	var foxytunesKeys = new Array();
	var mainKeySet = document.getElementById('mainKeysetFoxyTunes');
	var keyElements = mainKeySet.getElementsByTagName('key');
	for (var i = 0; i < keyElements.length; i++) {
		var keyElement = keyElements[i];
		var id = keyElement.id;
		try {
			var idArray = id.split("-");
			if (idArray[0].toLowerCase() != 'foxytunes') {
				continue;
			}
		} catch(e) {
			continue;
		}
 
		//dump("Found FoxyTunes key: " + id + "\n");
		foxytunesKeys.push(keyElement);
	}
	return foxytunesKeys;
}

function foxytunesOverrideKeyIfNeeded(keyElem) {

	if (foxytunesGetPlatform() != 'mac') {
		// currently, overrides are only for mac
		return;
	}

	if (keyElem.hasAttribute('origmackey')) {
		var keyOrigMac = keyElem.getAttribute('origmackey');
		keyElem.setAttribute('origkey', keyOrigMac);
	}

	if (keyElem.hasAttribute('origmackeycode')) {
		var keyCodeOrigMac = keyElem.getAttribute('origmackeycode');
		keyElem.setAttribute('origkeycode', keyCodeOrigMac);
	}

	if (keyElem.hasAttribute('origmacmodifiers')) {
		var modifiersOrigMac = keyElem.getAttribute('origmacmodifiers');
		keyElem.setAttribute('origmodifiers', modifiersOrigMac);
	}
}

function foxytunesInitKeyboardShortcuts() {
	var keys = foxytunesGetAllKeys();

	var bUseShortcuts = foxytunesReadSinglePreference('use_shortcuts', 
								foxytunesShortcutsEnabledByDefault());

	for (var i = 0; i < keys.length; i++) {
		var keyElem = keys[i];

		if ((keyElem.getAttribute('keyenabled') == 'false') || (bUseShortcuts != 'true')) {
			if (keyElem.hasAttribute('command')) {
				keyElem.setAttribute('origcommand', keyElem.getAttribute('command'));
				keyElem.removeAttribute('command');
			}

			if (keyElem.hasAttribute('oncommand')) {
				keyElem.setAttribute('origoncommand', keyElem.getAttribute('oncommand'));
				keyElem.removeAttribute('oncommand');
			}
		}

		// change the defaults if needed for different platforms
		foxytunesOverrideKeyIfNeeded(keyElem);

		var key = keyElem.getAttribute('key');
		var keyCode = keyElem.getAttribute('keycode');
		var modifiers = keyElem.getAttribute('modifiers');

		if (!key && !keyCode) {
			// we have to initialize this shortcut
			var keyOrig = keyElem.getAttribute('origkey');
			var keyCodeOrig = keyElem.getAttribute('origkeycode');
			var modifiersOrig = keyElem.getAttribute('origmodifiers');

			keyElem.setAttribute('modifiers', modifiersOrig);

			if (keyOrig) {
				keyElem.setAttribute('key', keyOrig);
			} else {
				keyElem.setAttribute('keycode', keyCodeOrig);
			}
		} 
	}
}

function foxytunesOpenConfigureShortcutsDialog(bReadOnly) {
  window.openDialog('chrome://foxytunes/content/foxytunesShortcuts.xul',
							'',
							'chrome,centerscreen,modal',
							foxytunesGetAllKeys(), bReadOnly);
}

function foxytunesGoToURL(url) {
  if (!foxytunesIsInThunderbird()) {
	// Firefox
	if (gbFoxyTunesOpenWindowInTab) {
		var newTab = window.getBrowser().addTab(url);
	        window.getBrowser().selectedTab = newTab;
	} else {
		window.open(url);
	}
  } else {
	// Thunderbird
	foxytunesLaunchExternalURLFromThunderbird(url);
  }
}

function foxytunesSubscribeToNewsletter(url, title) {
  var email = prompt(title, '');
  if ((email != null) && (email != '')) {
	url += '?operation=subscribe&user_email=' + escape(email);
  	foxytunesGoToURL(url);
  }
}

function foxytunesSetPlayerOptions() {
   if (gFoxyTunesPlayerObj != null) {
	try {
   		var home = foxytunesGetHome();
		var bCharaterEncodingOn = (gFoxyTunesCharacterEncoding != 'default');
		var options = gFoxyTunesCurrentPlayerOptions;
		options += " -home " + home.path;
		if (bCharaterEncodingOn) {
			options += " -encoded_chars ";
		}

		gFoxyTunesPlayerObj.SetOptions(options);
	}
	catch (err) {
		dump("\nCouldn't set player options: " + err + "\n");
	}
   }
}

function foxytunesSetPlayerOptionsPreset(event) {
    var item = event.target;
    gFoxyTunesCurrentPlayerOptions = item.value;
    foxytunesWritePreferences();
    foxytunesSetPlayerOptions(); // send the options to the player (null protected)
}

function foxytunesSetPlayerOptionsCustom(event, title) {
    var options = prompt(title, gFoxyTunesCustomPlayerOptions);
    if (options != null) {
	  gFoxyTunesCustomPlayerOptions = options;
	  gFoxyTunesCurrentPlayerOptions = options;
	  foxytunesWritePreferences();
	  foxytunesSetPlayerOptions(); // send the options to the player (null protected)
    }
}

function foxytunesRegisterGlobalScrollEvent(event) {
    if (event.type == "mouseover") {
//	dump('Added global scroll listener\n');
	window.addEventListener("DOMMouseScroll", foxytunesScrollOnVolumeControls, false);
    } else {
//	dump('Removed global scroll listener\n');
	window.removeEventListener("DOMMouseScroll", foxytunesScrollOnVolumeControls, false);
    }
}

function foxytunesRegisterVolumeSliderEvents() {
   var volumeSlider = document.getElementById('foxytunes-volume-slider'); 
   volumeSlider.addEventListener("DOMAttrModified", foxytunesVolumeSliderChanged, false);
   var platform = foxytunesGetPlatform();

   var targetIDs = ['foxytunes-all-buttons-container-statusbar', 'foxytunes-main-menu-statusbar'];
   for (var i = 0; i < targetIDs.length; i++) {
	  var targetElem = document.getElementById(targetIDs[i]);
	  if (platform != 'linux') {
		  targetElem.addEventListener("DOMMouseScroll", foxytunesScrollOnVolumeControls, false);
	  } else {
  		 // on Linux the above doesn't seem to work, the scroll needs to be a global window event
		 targetElem.addEventListener("mouseover", foxytunesRegisterGlobalScrollEvent, false);
		 targetElem.addEventListener("mouseout", foxytunesRegisterGlobalScrollEvent, false);
	  }
   }
}

function foxytunesInitVolumeSlider() {
   foxytunesRegisterVolumeSliderEvents();

   // see if we should actualy query the player for the actual volume on startup and update
   // the slider position, or defer this operation until the first mouse over (default)
   var confItem = document.getElementById('foxytunes-configuration-update-volume-on-startup');
   if (confItem.getAttribute('checked') == 'true') {
           foxytunesRefreshVolumeSliderPosition();
   } else {
	   foxytunesVolumeSliderPositionDirty("true");
   }
}

function foxytunesInsertPlayersMenuItem(label) {
   var popup = document.getElementById('foxytunes-available-player-popup');

   var str = foxytunesGetLocaleString(label);

   // replace 'Firefox' with the correct host if needed
   str = foxytunesAdjustStringForCorrectHost(str);

   var newElement;
   newElement = document.createElement('menuitem');
   newElement.setAttribute('type', 'radio');
   newElement.setAttribute('name', 'foxytunes-player');
   newElement.setAttribute('label', str);
   newElement.setAttribute('tooltiptext', str);
   newElement.setAttribute('checked', 'true');
   popup.appendChild(newElement);
}

function foxytunesGetProfileDir() {
   var dirServiceProp = Components.classes["@mozilla.org/file/directory_service;1"];
   dirServiceProp = dirServiceProp.getService(Components.interfaces.nsIProperties);
   var profileDir = dirServiceProp.get("ProfD", Components.interfaces.nsIFile);
   return profileDir;
}

function foxytunesGetHome() {
   var profileDir = foxytunesGetProfileDir();
   var foxyTunesHome = profileDir.clone();
   foxyTunesHome.append('extensions');
   foxyTunesHome.append('{463F6CA5-EE3C-4be1-B7E6-7FEE11953374}');

   return foxyTunesHome;
}

function foxytunesGetComponentFile(filename) {
   var home = foxytunesGetHome();
   var componentFile = home.clone();
   componentFile.append('components');
   componentFile.append(filename);
   return componentFile;
}

function foxytunesRemoveRegistryFiles() {
   var profileDir = foxytunesGetProfileDir();
   var compregFile = profileDir.clone();
   compregFile.append('compreg.dat');
   compregFile.remove(false);
   dump("\nRemoved " + compregFile.path + "\n");

   var xptiFile = profileDir.clone();
   xptiFile.append('xpti.dat');
   xptiFile.remove(false);
   dump("\nRemoved " + xptiFile.path + "\n");
}

function foxytunesGetPlatform() {
	var platform = navigator.platform.toLowerCase();
	
	if (platform.indexOf('linux') != -1) {
		return 'linux';
	}

	if (platform.indexOf('mac') != -1) {
		return 'mac';
	}

	return 'windows';
}

function foxytunesInstallPlatformSpecificLibraryIfNeeded() {
   if (foxytunesIsInMozilla()) {
	// we are in Mozilla, installed properly through install.js
	return false;
   }

   var platform = foxytunesGetPlatform();
   
   if (platform == 'windows') {
		// the default dll is for windows
		return false;
   }

   var platformSpecificFileName = 'FoxyTunes.dll.' + platform;

   var foxyTunesDllPlatformSpecificFile = 
			foxytunesGetComponentFile(platformSpecificFileName);

   if (!foxyTunesDllPlatformSpecificFile.exists()) {
	// the file doesn't exist, we already renamed it
 	return false;
   }

   // the file exists, rename it, so it becomes the main library file
   dump("\nRenaming " + platformSpecificFileName + " to FoxyTunes.dll\n");
   foxyTunesDllPlatformSpecificFile.moveTo(null, 'FoxyTunes.dll');

   // re-get the file (because it now points to the new name):
   foxyTunesDllPlatformSpecificFile = 
		foxytunesGetComponentFile(platformSpecificFileName);

   if (foxyTunesDllPlatformSpecificFile.exists()) {
	// the file still exists, something's wrong
	dump("\nFailed renaming " + platformSpecificFileName + " to FoxyTunes.dll\n");
	return false; 
   }

   // load the dll so the user doesn't have to restart the browser to see the change
   var mainDLLFile = foxytunesGetComponentFile('FoxyTunes.dll');
   var componentRegistrar = Components.manager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
   try {
       componentRegistrar.autoRegister(mainDLLFile);
   } catch(err) {
     	dump("\nFailed autoRegistering renamed platform specific FoxyTunes.dll\n");
	return false; 
   }

   // the file existed, we have renamed (installed) it
   // remove compreg.dat and xpti.dat files, so on the next install
   // the file will get registered 
   // remark: with auto-registration above, this probably is no longer needed
   foxytunesRemoveRegistryFiles();

   return true;
}

function foxytunesFoxyTunesToBeInstalled() {
   var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
             .getService(Components.interfaces.nsIRDFService);
   var Container = Components.classes["@mozilla.org/rdf/container;1"]
             .getService(Components.interfaces.nsIRDFContainer);
   var extensionDS= Components.classes["@mozilla.org/extensions/manager;1"]
	      .getService(Components.interfaces.nsIExtensionManager).datasource;
   var root = RDFService
             .GetResource("urn:mozilla:extension:root");
   var nameArc = RDFService
             .GetResource("http://www.mozilla.org/2004/em-rdf#name");
   var toBeInstalledArc = RDFService
             .GetResource("http://www.mozilla.org/2004/em-rdf#toBeInstalled");

   // This seems to fail sometimes...
   // Could it be that the EM datasource isn't initialized yet 
   // because it's being initialized along with us (or after us)?
   Container.Init(extensionDS,root);

   var found = false;
   var elements = Container.GetElements();
   while (elements.hasMoreElements()) {
	var element = elements.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
	var name = "";
	var toBeInstalled = "";

	var target = extensionDS.GetTarget(element, nameArc ,true);
	if (target) {
		name = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
 	}

        target = extensionDS.GetTarget(element, toBeInstalledArc ,true);
	if (target) {
        	toBeInstalled=target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
	}
    
	if (toBeInstalled && (toBeInstalled == "true") && (name == "FoxyTunes")) {
		found = true;
	}
   } 
   return found;
}

function foxytunesRenameMainDll() {
   var foxyTunesDll = foxytunesGetComponentFile('FoxyTunes.dll');
   if (!foxyTunesDll.exists()) {
	// the file doesn't exist, something unexpected happend 
 	return false;
   }

   // the file exists, rename it, so it the install can succeed next
   // time firefox is started
   dump("\nRenaming FoxyTunes.dll to FoxyTunes.dll.uninstalled\n");
   foxyTunesDll.moveTo(null, 'FoxyTunes.dll.uninstalled');
   // re-get the file (because it now points to the new name):
   foxyTunesDll = foxytunesGetComponentFile('FoxyTunes.dll');
   if (foxyTunesDll.exists()) {
	// the file still exists, something's wrong 
	dump("\nFailed renaming FoxyTunes.dll to FoxyTunes.dll.uninstalled\n");
	return false; 
   }

   return true;
}

function foxytunesCheckForFailedUpgrade() {
   try {
     if (foxytunesFoxyTunesToBeInstalled()) {
  	  var upgradePrompt = foxytunesGetLocaleString("failedUpgradePrompt");
    	  if (confirm(upgradePrompt)) {
		if (foxytunesRenameMainDll()) {
			var restartPrompt = foxytunesGetLocaleString("failedUpgradeRestartFirefoxPrompt");
			restartPrompt = foxytunesAdjustStringForCorrectHost(restartPrompt);
			alert(restartPrompt);
		} else {
			var reinstallPrompt = foxytunesGetLocaleString("failedUpgradeReinstallFoxyTunesPrompt");
			alert(reinstallPrompt);
		}
 	  }
     }
  } catch (err) {
	dump('Problem checking for failed upgrade: ' + err + '\n');
  }
}

function foxytunesThunderbirdSpecificInit() {
   if (!foxytunesIsInThunderbird()) {
	// this is not Thunderbird
	return;
   }

   // do something Thunderbird specific here:
}

function foxytunesShowRestartBrowserAlert() {
   var str = foxytunesGetLocaleString('pleaseRestartFirefoxAlert');
   // replace 'Firefox' with the correct host if needed
   str = foxytunesAdjustStringForCorrectHost(str);
   alert(str);
}

function foxytunesRestoreIfHidden(event, prompt) {

	if (!confirm(prompt)) {
		return; 
	}

	var foxyTunes = document.getElementById('foxytunes-statusbar-panel');
	foxyTunes.setAttribute('hidden', 'false');	

	// hide the "restore" button
	event.target.setAttribute('hidden', 'true');
}

function foxytunesVerifyWidth() {
	var foxyTunes = document.getElementById('foxytunes-statusbar-panel');
	var statusBar = document.getElementById('status-bar');
	var doc = document.documentElement;

	var foxyTunesWidth = foxyTunes.boxObject.width;
	var statusBarWidth = statusBar.boxObject.width;
	var windowWidth = doc.boxObject.width; 

	var restoreFoxyTunesButton = document.getElementById('foxytunes-statusbar-restore-foxytunes');

	if (windowWidth < 2) {
		// seems that this function was called too soon (no width yet)
		return;
	}

//	dump('Status bar width:' + statusBarWidth + ", window width: " + windowWidth + "\n");

	var bOverflowHidden = (foxyTunes.getAttribute('foxy-overflow-hidden') == 'true');

	// the status bar should never be wider than the documents (window)
	// hide FoxyTunes
	// if bOverflowHidden is true, don't hide (this means that visibility was forced)
	if ((statusBarWidth > windowWidth) && !bOverflowHidden) {
		dump('Status bar is too wide (' + statusBarWidth + " > " + windowWidth + "). Hiding FoxyTunes...\n");

		foxyTunes.setAttribute('hidden', 'true');
		foxyTunes.setAttribute('foxy-overflow-hidden', 'true');
		restoreFoxyTunesButton.setAttribute('hidden', 'false');

		gFTLastWindowWidth = windowWidth;
		gFTLastFoxyTunesWidth = foxyTunesWidth;
	} else {
		if  (bOverflowHidden) {
			// did the window grow wider since we last hidden FoxyTunes?
			var bDocGotWider = (windowWidth > gFTLastWindowWidth);

			// would FoxyTunes fit in the window if we showed it?
			// add some width to foxytunes to make sure it fits
			var bEnoughSpace = (windowWidth > (gFTLastFoxyTunesWidth + 250));

			if (bDocGotWider && bEnoughSpace) {
				foxyTunes.setAttribute('hidden', 'false');	
				foxyTunes.setAttribute('foxy-overflow-hidden', 'false');
				restoreFoxyTunesButton.setAttribute('hidden', 'true');
				dump("The browser is now wide enough. Restoring FoxyTunes...\n");
			}
		}
	}
}

function foxytunesOnBrowserStatusChanged(event) {
	if (event.attrName != 'label') {
		// only label changes interest us
		return;
	}

	var foxytunesWidget = document.getElementById('foxytunes-statusbar-panel');
	var bFoxyTunesIsOnStatusBar = foxytunesWidget.parentNode.tagName == 'statusbar';

	if (!bFoxyTunesIsOnStatusBar) {
		// FT is not on the status bar - hiding it won't help
		return;
	}

	var bHiddenToRevealStatus = foxytunesWidget.getAttribute('foxy-see-status-hidden') == 'true';

	// proceed only if:
	// 1. shift is pressed - consider hiding FT or
	// 2. FT is hidden - consider revealing it
	if (!gbFoxyTunesShiftDown && !bHiddenToRevealStatus) {
		 return;
	}

	var statusElem = event.target;
	var width = statusElem.boxObject.width;

	// temporarily remove flex to measure the real width (shouldn't cause a re-flow...)
	var oldFlex = statusElem.getAttribute('flex'); 
	statusElem.setAttribute('flex', '0');
	var fullWidth = statusElem.boxObject.width;
	statusElem.setAttribute('flex', oldFlex);

	//dump('Status width - current: ' + width + ", full: " + fullWidth + '\n');

	if ((fullWidth > width) && gbFoxyTunesShiftDown) {
		// the full width is bigger than the current width, hide if not already hidden
		if (!bHiddenToRevealStatus) {
			foxytunesWidget.setAttribute('hidden', 'true');
			foxytunesWidget.setAttribute('foxy-see-status-hidden', 'true');
		}
	} else {
		// the full width is smaller than the current width, reveal if hidden
		// don't reveal if the shift is still down
		if (bHiddenToRevealStatus && !gbFoxyTunesShiftDown) {
			foxytunesWidget.setAttribute('hidden', 'false');
			foxytunesWidget.setAttribute('foxy-see-status-hidden', 'false');
		}
	}
}

function foxytunesOnMouseMove(event) {
	var bOldShiftDown = gbFoxyTunesShiftDown;
	gbFoxyTunesShiftDown = event.shiftKey;

	// shift state changed, trigger DOMAttrModified on the main status display
	if (gbFoxyTunesShiftDown != bOldShiftDown) {
		// trigger DOMAttrModified on the main status display
		var statusbarDisplay = document.getElementById('statusbar-display');
		var status = statusbarDisplay.getAttribute('label');
		statusbarDisplay.setAttribute('label',  status + " ");
		statusbarDisplay.setAttribute('label',  status);
	}
}

function foxytunesInitOverlay() {
 try {

   try {
	   foxytunesInitKeyboardShortcuts();
   } catch(err) {
	   // let's assume that shortcut related problems are non-critical
	   dump("Error initializing shortcuts " + err + "\n");
   }

   if (!foxytunesIsInMozilla()) {
	   // do this only in Firefox/Thunderbird (extension manager):
	   foxytunesCheckForFailedUpgrade();
   }
   // proceed even if the failed upgrade detected, the user works with the old version 
   foxytunesReadPreferences();
   foxytunesAddonSetCurrentPlayerIcon();
   foxytunesInstallPlatformSpecificLibraryIfNeeded();

   foxytunesPopulateCharacterEncodings();
   var nPlayers = foxytunesPopulatePlayers();
   foxytunesPopulateRecentCharsets();
   foxytunesInitVolumeSlider();

   if (nPlayers == 0)  {
	foxytunesInsertPlayersMenuItem('noPlayerModulesFound');
   }

   // the following will re-install listeners (volume slider...):	
   foxytunesSetFoxyTunesPosition();

   foxytunesThunderbirdSpecificInit();

   foxytunesInitTrackTitleLabel();

   // make sure FoxyTunes doesn't cause width overflows in the browser window:
   foxytunesVerifyWidth();
   addEventListener("resize", foxytunesVerifyWidth, false);

   // hide FoxyTunes when the status display label is cropped and shift is pressed:
   if (!foxytunesIsInThunderbird() && 
		(foxytunesReadSinglePreference('statusbar_reveal_disabled', 'false') != 'true'))  {
	var statusbarDisplay = document.getElementById('statusbar-display');
	statusbarDisplay.addEventListener("DOMAttrModified", foxytunesOnBrowserStatusChanged, false);   
	addEventListener("mousemove", foxytunesOnMouseMove, false);
   }
 } catch(err) {
   dump("Fatal error initializing FoxyTunes: " + err + "\n");
 }
}

addEventListener("load", foxytunesInitOverlay, false);

// preferences:
var gFoxyTunesPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
var gFoxyTunesPref = gFoxyTunesPrefService.getBranch("foxytunes."); 

// unicode converter:
var foxytunesScriptableUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].getService();
var gFoxyTunesUnicodeConverter = foxytunesScriptableUnicodeConverter.QueryInterface(
				         Components.interfaces.nsIScriptableUnicodeConverter);

// current charater encoding for the media information:
//var gFoxyTunesCharacterEncoding = 'default';
var gFoxyTunesCharacterEncoding;

// track info display timer ID:
var gFoxyTunesTrackInfoTimerID = 0;

// playing global flag - if track info tooltip is displayed and not playing, stop the timer:
var gbFoxyTunesPlaying = false;

// changing slider position global flag, if set, then changing the curpos property won't cause SetVolume
var gbFoxyTunesChangingSliderPos = false;

// array of menuitems of all available charsets 
var gFoxyTunesAllCharSets; 

// list of the recent charsets (loaded from pref)
var gFoxyTunesRecentCharsets;

// maximal number of recent charsets:
var gFoxyTunesMaxRecentCharsets;

// custom player options string:
var gFoxyTunesCustomPlayerOptions;

// current player options string:
var gFoxyTunesCurrentPlayerOptions;

// whether to open windows in tabs (as opposed to new windows):
var gbFoxyTunesOpenWindowInTab = false;

// the id of the DOM element to position FoxyTunes at:
var gFoxyTunesParentElementID = '';

// the id of the DOM element to insert FoxyTunes before
// if equals to the parent ID, insert as the last child:
var gFoxyTunesInsertBeforeElementId = '';

// if the insert before fails, try inserting after this one:
var gFoxyTunesInsertAfterElementId = '';

// current class name (contract ID) for the player XPCOM object:
var gFoxyTunesCurrentPlayerClass;

// the player controller object:
var gFoxyTunesPlayerObj = null;

// this will cache the size of the track info tooltip
// solves a positioning problem - we do not know the height
// before the first popup. After the first popup it is queried and cached
var gFoxyTunesTrackInfoTooltipHeight = 0;

// cached current track title from the last query to the player:
var gFoxyTunesCurrentTrackTitle = null;

// time ID for auto-hide:
var gFoxyTunesAllButtonsAutohideTimerID = 0;

// auto hide timeout:
var gFoxyTunesAutoHideTimeout = 1000;

// don't update track title on prev/next/play
var gbFoxyTunesDontUpdateTitleOnCommand = false;

// update track title on prev/next/play timeout
var gFoxyTunesUpdateTitleOnCommandTimerID = 0;

// override the regular track title popup (show a regular tooltip)
var gbFoxyTunesNoTitlePopup = false;

// monitors the shift key state
var gbFoxyTunesShiftDown = false;

// URL unescape titles
var gbURLUnescape = false;


function foxytunesAddonNextFavoritePlayer() {
	
	var favoritePlayersArray = ["WinAmp", "YahooMusicEngine", "iTunes"];
	var testClassContract;
	var currindex = -1;
	for(var i = 0; i<favoritePlayersArray.length; ++i) {
		testClassContract = "@foxytunes.org/FoxyTunes/" + favoritePlayersArray[i] + ";1"
		if(testClassContract == gFoxyTunesCurrentPlayerClass)
			currindex = i;
	}
	currindex++;
	if(currindex == favoritePlayersArray.length)
		currindex = 0;
		
	
	var classContract = "@foxytunes.org/FoxyTunes/" + favoritePlayersArray[currindex] + ";1"
	gFoxyTunesCurrentPlayerClass = classContract;
    foxytunesWritePreferences();
    
    // invalidate the global object. it will be initialized on next operation
    gFoxyTunesPlayerObj = null;
    
    foxytunesAddonSetCurrentPlayerIcon();
    foxytunesUpdateTrackTitleLabel(true);
	
}

function foxytunesAddonSetCurrentPlayerIcon() {
	
	var newPlayerName = foxytunesTryGetFoxyTunesPlayer(gFoxyTunesCurrentPlayerClass).toLowerCase();
    document.getElementById("foxytunesAddon-PlayerIcon-statusbar").setAttribute("playername", newPlayerName);
	
}