/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Contributor(s):
 *    Devon Jensen <velcrospud@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function foxytunesGetTrackInfoLabelElement() {
	return document.getElementById('foxytunes-track-title-status-label');
}

function foxytunesInitTrackTitleLabel() {

	// try getting polling interval from the preferences
	try {
		gFoxyTunesTrackTitleQueryInterval = gFoxyTunesPref.getIntPref("title_poll_interval");
	} catch (e) {} 

//	var str = foxytunesGetLocaleString("idle");
	var str = 'FoxyTunes';
	foxytunesSetCurrentTrackTitleLabel(str);
	if (foxytunesTrackTitleDisplayVisibile()) {
		// see if we should update the track info on startup (connect to the player)
		var confItem = document.getElementById('foxytunes-configuration-update-track-title-on-startup');
		if (confItem.getAttribute('checked') == 'true') {
			foxytunesUpdateTrackTitleLabel(true);
		}

		// start the polling
		foxytunesTrackTitleLabelUpdater();
	} else {
		var labelElem = foxytunesGetTrackInfoLabelElement();
		// make sure there is no scrolling if hidden
		labelElem.stopScroll();
	}
}

function foxytunesUpdateTrackTitleLabel(connect) {
	// query player only if asked to connect (param) or already connected
	if (connect || (gFoxyTunesPlayerObj != null)) {
		var title = foxytunesGetCurrentTrackTitle();
		foxytunesSetCurrentTrackTitleLabel(title);
		foxytunesUpdateTrackPositionMarker();
	} else {
		//var str = foxytunesGetLocaleString("idle");
		var str = 'FoxyTunes';
		foxytunesSetCurrentTrackTitleLabel(str);
	}
}

function foxytunesUpdateTrackPositionMarker() {
	var elem = foxytunesGetTrackInfoLabelElement();

	if (elem.getAttribute('seekslider') != 'true') {
		// we do not want a seek slider
		elem.progress = -1;
		return;
	}

	var length = foxytunesGetCurrentTrackItem('Length');
	var pos = foxytunesGetCurrentTrackItem('Position');
        if ((length != null) && (pos != null)) {
		elem.progress = pos / length;
	} else {
		elem.progress = -1;
	}
}

function foxytunesTrackTitleLabelUpdater() {
	//dump("\nquering title\n");
	foxytunesUpdateTrackTitleLabel();

	gFoxyTunesTrackTitleDisplayTimerID = setTimeout('foxytunesTrackTitleLabelUpdater()', 
								gFoxyTunesTrackTitleQueryInterval);
}

function foxytunesSetCurrentTrackTitleLabel(title) {
        if (title == null) {
		return;
      	}

	if (title == '') {
	    	var str = foxytunesGetLocaleString("idle");
		title = str;
	}

	var elem = foxytunesGetTrackInfoLabelElement();

	if(elem.getAttribute("label") != title) {
		elem.label = title;
		foxytunesAddonSetNewTrackTitleWidth(elem);
	}
}

function foxytunesTrackTitleDisplayVisibile() {
	var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
	var visible = (trackTitleDisplay.getAttribute('hidden') == 'false');
	return visible;
}

function foxytunesTrackInfoDisplayMouseOver() {
  // cancel all timers
  if (gFoxyTunesTrackInfoDisplayAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesTrackInfoDisplayAutohideTimerID);
  }
}

function foxytunesTrackInfoDisplayMouseOut() {
  if (! foxytunesTrackTitleAutoHideIsOn()) {
  	// auto-hide is off, nothing to do
	return;
  }

  var arrowButton = document.getElementById('foxytunes-toggle-track-title-display-visibility'); 
  var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
  var bHidden = (trackTitleDisplay.getAttribute('hidden') == 'true');

  if (bHidden) {
	// already hidden, nothing to do
	return;
  }

  // stop the previous timers if exist
  if (gFoxyTunesTrackInfoDisplayAutohideTimerID != 0) {
	clearTimeout(gFoxyTunesTrackInfoDisplayAutohideTimerID);
  }

  // schedule auto-hide
  gFoxyTunesTrackInfoDisplayAutohideTimerID = 
			setTimeout(foxytunesToggleObjectVisibilityWithArrow, gFoxyTunesAutoHideTimeout, arrowButton, trackTitleDisplay, true);
}


function foxytunesTrackTitleAutoHideIsOn() {
  var autoHideMenuItem = document.getElementById('foxytunes-track-info-panel-auto-hide-toggle');
  return (autoHideMenuItem.getAttribute('checked') == 'true');
}

function foxytunesToggleTrackTitleDisplayVisibility(event, bHovering) {
	var arrowButton = event.target; 
	var trackTitleDisplay = document.getElementById('foxytunes-track-title-display-container-statusbar'); 
        var bHidden = (trackTitleDisplay.getAttribute('hidden') == 'true');

  	// if hovering and the panel is already visible, or auto-hide is off, return
	if (bHovering && (!bHidden || !foxytunesTrackTitleAutoHideIsOn())) {	
		return;
	}

	foxytunesToggleObjectVisibilityWithArrow(arrowButton, trackTitleDisplay, true);

	// stop any old timers
	if (gFoxyTunesTrackTitleDisplayTimerID != 0) {
		clearTimeout(gFoxyTunesTrackTitleDisplayTimerID);
	}

	var labelElem = foxytunesGetTrackInfoLabelElement();

	// start updater if needed
	if (foxytunesTrackTitleDisplayVisibile()) {
		// refresh title, connect if needed
		foxytunesUpdateTrackTitleLabel(true);

		// start polling
		foxytunesTrackTitleLabelUpdater();

		// start scrolling if needed
		labelElem.refreshScroll(); 
	} else {
		// stop (pause) the scrolling
		labelElem.stopScroll(); 
	}
	foxytunesAddonSetNewTrackTitleWidth(labelElem);
}


function foxytunesTrackTitleCopyToClipBoard() {
	var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"];
	clipboardHelper = clipboardHelper.getService(Components.interfaces.nsIClipboardHelper);

	var labelElem = foxytunesGetTrackInfoLabelElement();
     	clipboardHelper.copyString(labelElem.getAttribute('label'));
}

function foxytunesTrackTitleToggleScrolling() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var scrolling;
	if (labelElem.getAttribute('scrolling') == 'true') {
		scrolling = 'false';
	} else {
		scrolling = 'true';
	}

	labelElem.setAttribute('scrolling', scrolling);
	labelElem.scrolling = (scrolling == 'true');

	labelElem.resetPosition();

	// start/stop scrolling if needed
	labelElem.refreshScroll(); 
}

function foxytunesTrackToggleSeekSlider() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var seekslider;
	if (labelElem.getAttribute('seekslider') == 'true') {
		seekslider = 'false';
	} else {
		seekslider = 'true';
	}

	labelElem.setAttribute('seekslider', seekslider);

	foxytunesUpdateTrackPositionMarker();
}

function foxytunesTrackTitleToggleAlignment() {
	var labelElem = foxytunesGetTrackInfoLabelElement();
	var align;
	if (labelElem.getAttribute('labelalign') == 'right') {
		align = 'left';
	} else {
		align = 'right';
	}

	labelElem.setAttribute('labelalign', align);

	labelElem.resetPosition();
}

function foxytunesTrackTitleDisplayResizeDown(event) {
	if (gbFoxyTunesTrackPressing) {
		// sanity check
		dump("\nMouse down event when already pressing\n");
		return;
	}

	gbFoxyTunesTrackPressing = true;
	gFoxyTunesResizeStartX = event.screenX;
	var labelElem = foxytunesGetTrackInfoLabelElement();
	gFoxyTunesResizeInitialWidth = labelElem.getAttribute('width');
	addEventListener("mouseup", foxytunesTrackTitleDisplayResizeUp, false);
	addEventListener("mousemove", foxytunesTrackTitleDisplayResizeMove, false);
	dump("Down: " + gFoxyTunesResizeStartX + "\n");
}

function foxytunesTrackTitleDisplayResizeUp(event) {
	if (gbFoxyTunesTrackPressing) {
		gbFoxyTunesTrackPressing = false;
		removeEventListener("mouseup", foxytunesTrackTitleDisplayResizeUp, false);
		removeEventListener("mousemove", foxytunesTrackTitleDisplayResizeMove, false);
		var labelElem = foxytunesGetTrackInfoLabelElement();
		labelElem.resetPosition();
		foxytunesUpdateTrackPositionMarker();
	}
	dump("Up\n");
}

function foxytunesTrackTitleDisplayResizeMove(event) {
	if (gbFoxyTunesTrackPressing) {
		var offset = gFoxyTunesResizeStartX - event.screenX;
		var newWidth = gFoxyTunesResizeInitialWidth*1 + offset*1;
		//dump("move " + offset + ": ");
		dump("gFoxyTunesResizeInitialWidth " + gFoxyTunesResizeInitialWidth + " - ");
		//dump("newWidth " + newWidth + "\n");
		if (newWidth*1 > 10) {
			var labelElem = foxytunesGetTrackInfoLabelElement();
			labelElem.setAttribute('width', newWidth);	
		}
	}
}

var foxytunesTrackTitleDragStartObserver =
{
  onDragStart: function (event, transferData, action) {
	var labelElem = foxytunesGetTrackInfoLabelElement();
     	var label = labelElem.getAttribute('label');

	foxytunesInstallDragDropObservers();
	transferData.data = new TransferData();
	transferData.data.addDataForFlavour("text/unicode", label);
  }
}

// the time ID for the query player for title polling:
var gFoxyTunesTrackTitleDisplayTimerID = 0;

// track title polling interval:
var gFoxyTunesTrackTitleQueryInterval = 5000;

// mouse pressed flag (resizer):
var gbFoxyTunesTrackPressing = false; 

// starting X coordinate for the resizer:
var gFoxyTunesResizeStartX = false; 

// the initial track title label width (for the resizer):
var gFoxyTunesResizeInitialWidth = false; 

// time ID for auto-hide:
var gFoxyTunesTrackInfoDisplayAutohideTimerID = 0;


function foxytunesAddonSetNewTrackTitleWidth(trackTitleLabelElem) {
	
	var newLabelWidth = trackTitleLabelElem.labelElem.boxObject.width + 2;  // got from dom inspector - javascript props
	// min width - so that track seeking works okay even for small names
	// Add pref for minwidth?
	if (newLabelWidth < 60)
		newLabelWidth = 60;
	
	trackTitleLabelElem.setAttribute('width', newLabelWidth);
}
