/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function foxytunesIsInFirefox() {
	return (navigator.userAgent.search(/Firefox/gi) != -1);
}

function foxytunesIsInThunderbird() {
	return (navigator.userAgent.search(/Thunderbird/gi) != -1);
}

function foxytunesIsInMozilla() {
	return (!foxytunesIsInFirefox() && !foxytunesIsInThunderbird());
}


function foxytunesAdjustStringForCorrectHost(str) {
	if (foxytunesIsInThunderbird()) {
		str = str.replace('Firefox', 'Thunderbird');
	}

	if (foxytunesIsInMozilla()) {
		str = str.replace('Firefox', 'Mozilla');
	}
	return str;
}

function foxytunesLaunchExternalURLFromThunderbird(url) {
        var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();         
        messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
        messenger.launchExternalURL(url);         
}

function foxytunesGoToURLAndClose(url) {
	if (!foxytunesIsInThunderbird()) {
		// Firefox:
		var foxyTunesPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
		var foxyTunesPref = foxyTunesPrefService.getBranch("foxytunes."); 
		var bFoxyTunesOpenWindowInTab;
	   	try {
	     		bFoxyTunesOpenWindowInTab = foxyTunesPref.getBoolPref("open_window_in_tab");
	   	} catch (err) {
			bFoxyTunesOpenWindowInTab = false;
		}

		if (bFoxyTunesOpenWindowInTab) {
			if (window.opener.getBrowser) {
				// we were opened by a browser
		        	var newTab = window.opener.getBrowser().addTab(url);
	       			window.opener.getBrowser().selectedTab = newTab;
			} else {
				// we were opened by the extensions window
	        		window.opener.open(url);
			}
	        } else {
	        	window.opener.open(url);
	        }
	} else { // !foxytunesIsInThunderbird()
		// Thunderbird
		foxytunesLaunchExternalURLFromThunderbird(url);
	}

    	window.close();
}

function foxytunesShowAlert(parent, title, text) {
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"];
	promptService = promptService.getService(Components.interfaces.nsIPromptService);
	promptService.alert(parent, title, text);
}

function foxytunesShowPrompt(parent, title, text, initialValue) {
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"];
	promptService = promptService.getService(Components.interfaces.nsIPromptService);

	if (!initialValue) {
		initialValue = "";
	}
	var answer = { value: initialValue };
    	var dummy = { value: false };
	var bOK = promptService.prompt(window, title, text, answer, null, dummy);

	if (bOK) {
		return answer.value;
	} else {
		return null;
	}
}


function foxytunesStringPadRight(str, length) {
	while (str.length < length) {
		str = str + " ";
	}
	return str;
}

function foxytunesGetLocaleString(strName, formattingValues) {
	var str = null;
	try {
		var strbundle = document.getElementById("foxytunes-strings");
		var strbundleFallback = document.getElementById("foxytunes-strings-fallback");
		if (formattingValues === undefined) {
            try {
    			str = strbundle.getString(strName);
            } catch(e) {
    			str = strbundleFallback.getString(strName);
            }
		} else {
            try {
    			str = strbundle.getFormattedString(strName, formattingValues);
            } catch(e) {
    			str = strbundleFallback.getFormattedString(strName, formattingValues);
            }
		}
	} catch (err) {
		dump("Couldn't get string: " + strName + "\n");
	}
	return str;
}

function foxytunesWriteSinglePreference(key, value) {
   var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
   var pref = prefService.getBranch("foxytunes."); 

   try {
     pref.setCharPref(key, value);
   } 
   catch (err) {
     dump("\nSetting preference for key " + key + " (" + value + ") failed: " + err + "\n");
   }
   try {
	prefService.savePrefFile(null);
   }
   catch (err) {
        dump("\nSaving preferences failed: " + err + "\n");
   }
}

function foxytunesReadSinglePreference(key, defaultVal) {
   var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
   var pref = prefService.getBranch("foxytunes."); 

   var val;
   try {
     val = pref.getCharPref(key);
   } 
   catch (err) {
     val = defaultVal; 
   }
   return val;
}

function foxytunesShortcutsEnabledByDefault() {
  if (foxytunesIsInThunderbird()) {
	return 'false';
  } else {
	return 'true';
  }
}

function foxytinesTrimString(string) {
	// collapse multiple white space characters into a single one:
	string = string.replace(/\s+/g, " ");

	// remove the leading space
	if (string.charAt(0) == ' ') {
		string = string.substr(1);
	}
		
	// remove the trailing space
	if (string.charAt(string.length - 1) == ' ') {
		string = string.substr(0, string.length - 1);
	}

	return string;
} 

